/**********************************************************************
 * Programacin Concurrente Y Distribuida
 * Ingeniera Informtica
 * 
 * Proyecto: Red Ferroviaria (Secuencial)
 * 
 * Fichero: Clase TramoFerroviario
 * 
 * Autor: J.M. Drake y L. Barros
 * Fecha: 31-09-09
 * 
 *********************************************************************/
/**
 * Representa un circuito que es seguido por un conjunto de trenes. 
 * Contiene  un centro de regulacin que controla su trfico y comparte
 * un tunel con otros tramos.
 */
class TramoFerroviario {
	 /**
	 * Definicin de constantes
	 */
 	final static int LONGITUD_TRAMO=96;	
    
	/**
	 * Estado de cada posicin de la red. Est libre si en ellas no est posicionado
	 * ningn tren. Los trenes ocupan una posicin cuando visualizan su posicin con
	 * situaTren() y la dejan libre cuando la borran con borraTren(). 
	 */
	public boolean[] posicionLibre;

	/**
	 * Da acceso a la red ferroviaria y a los servicios que ofrece.
	 */
	private RedFerroviaria laRed;
	
	/**
	 * Contiene la instancia del centro de regulacin que regula el trfico
	 * de trenes en el tramo ferroviario.
	 */
	private CentroRegulacion elCentroRegulacion;  	  
	/**
	* Tipo enumerado que designa los tramos ferroviarios existentes en
	* la red
	*/
   public enum Tramos {
		  TRAMO_ROJO,//Representa al tramo ferroviario Rojo
		  TRAMO_AZUL,//Representa al tramo ferroviario Azul
	}  
   /**
    * Identificador del tramo ferroviario que representa
    */
   TramoFerroviario.Tramos tramo;
  /**
   * Construye la instancias del tramos ferroviario. Se le proporciona
   * como  parmetro el acceso a la red ferroviaria en la que se instancia. 
   */
  public TramoFerroviario(Tramos tramo,RedFerroviaria laRed) {
	  this.laRed=laRed;
	  this.tramo=tramo;
	  posicionLibre= new boolean[longitudTramoFerroviario()+1]; 
	  elCentroRegulacion=new CentroRegulacion(laRed,tramo);	
	  for (int i=0;i<=longitudTramoFerroviario();i++){
		  posicionLibre[i]=true;		
	  }	 
  }
  /**
   * Retorna True si la posicin del tramo que se pasan comos parmetros 
   * no est ocupada por un tren. Se utiliza para evitar que los trenes
   * adelanten.Las posiciones con aparcamiento (Centros de regulacin y
   *  accesos al tunel) estn siempre libres, ya que en ellos se pueden 
   *  realizar adelantamientos. 
   */
 public boolean esPosicionLibre(int posicion){
	  if ((posicion==0)|
	      (posicion==(laRed.TUNNEL_RIGHT_POSITION-1))|
	      (posicion==laRed.LAST_POSITION)) {
		  return true;
	  }else {		  		 
		  return posicionLibre[posicion];		  
	  }
  }
  /**
   * Da acceso al centro de regulacin del tramo ferroviario.
   */
  public CentroRegulacion getCentroRegulacion() {
	  	return elCentroRegulacion;
  }
 
  /**
   * Retorna la longitud del tramo ferroviario
   */
  public int longitudTramoFerroviario(){	 
	  return LONGITUD_TRAMO; //==laRed.LAST_POSITION	 
  }
  /**
   * Retorna True si la posicin que se pasa corresponde a un tren situado
   * en el tunel.
   */
  public boolean enTunel(int posicion){
	  
	  return (posicion>=(laRed.TUNNEL_RIGHT_POSITION-1))&&(posicion<=(laRed.TUNNEL_LEFT_POSITION));
  }
  /**
   * Retorna True si la posicin corresponde al centro de regulacin.
   */  
  public boolean enCentroRegulacion(int posicion){
	  return (posicion==0)|| (posicion==laRed.LAST_POSITION);
  }
}
